<?php

namespace Uncanny_Automator_Pro\Integrations\Charitable;

/**
 * Class CHARITABLE_HELPERS
 *
 * @package Uncanny_Automator
 */
class CHARITABLE_HELPERS_PRO extends \Uncanny_Automator\Integrations\Charitable\CHARITABLE_HELPERS {

	/**
	 * CHARITABLE_HELPERS constructor.
	 */
	public function __construct() {
		parent::__construct();
	}

	/**
	 * Get Recurring Campaign Select.
	 *
	 * @param string $code
	 *
	 * @return array
	 */
	public function recurring_campaign_select( $code ) {
		return array(
			'input_type'  => 'select',
			'option_code' => $code,
			'label'       => __( 'Campaign', 'uncanny-automator' ),
			'required'    => true,
			'options'     => $this->get_recurring_campaign_options(),
		);
	}

	/**
	 * Get Recurring Campaign Options for Select.
	 *
	 * @return array
	 */
	public function get_recurring_campaign_options() {

		// Return cached options.
		static $options = null;
		if ( null !== $options ) {
			return $options;
		}

		// Build options.
		$options   = array();
		$campaigns = $this->get_recurring_campaigns();
		if ( ! empty( $campaigns ) ) {
			$options[-1] = _x( 'Any Recurring Campaign', 'Charitable', 'uncanny-automator-pro' );
			foreach ( $campaigns as $campaign ) {
				$options[ $campaign->ID ] = $campaign->post_title;
			}
		}

		return $options;
	}

	/**
	 * Get Recurring Campaign Posts By Meta.
	 *
	 * @return array
	 */
	public function get_recurring_campaigns() {

		$campaigns = \Charitable_Campaigns::query(
			array(
				'posts_per_page' => -1,
				'post_status'    => 'publish',
				'meta_query'     => array(
					'relation' => 'AND',
					array(
						'relation' => 'OR',
						array(
							'key'     => '_campaign_end_date',
							'value'   => gmdate( 'Y-m-d H:i:s' ),
							'compare' => '>=',
							'type'    => 'datetime',
						),
						array(
							'key'     => '_campaign_end_date',
							'value'   => 0,
							'compare' => '=',
						),
					),
					array(
						'key'     => '_campaign_recurring_donation_mode',
						'value'   => array( 'simple', 'advanced' ),
						'compare' => 'IN',
					),
				),
			)
		);

		return ! empty( $campaigns->posts ) ? $campaigns->posts : array();
	}

	/**
	 * Check if a campaign is recurring.
	 *
	 * @param int $campaign_id
	 *
	 * @return bool
	 */
	public function is_recurring_campaign( $campaign_id ) {
		$campaign_id = (int) $campaign_id;
		$campaigns   = $this->get_recurring_campaign_options();

		return array_key_exists( $campaign_id, $campaigns );
	}

	/**
	 * Check if a campaign is recurring and matches the given campaign ID.
	 *
	 * @param int $campaign_id
	 * @param int $check_campaign_id
	 *
	 * @return bool
	 */
	public function is_recurring_campaign_match( $campaign_id, $check_campaign_id ) {
		$campaign_id = (int) $campaign_id;
		if ( ! $this->is_recurring_campaign( $campaign_id ) ) {
			return false;
		}
		$check_campaign_id = (int) $check_campaign_id;
		if ( -1 === $check_campaign_id || $campaign_id === $check_campaign_id ) {
			return true;
		}

		return false;
	}

	/**
	 * Get Recurring Donation Tokens Config.
	 *
	 * @return array
	 */
	public function get_recurring_donation_tokens_config() {
		//REVIEW - should we just merge in all default tokens from $this->get_donation_tokens_config()
		return array(
			// Campaign tokens.
			array(
				'tokenId'   => 'CAMPAIGN_TITLE',
				'tokenName' => __( 'Campaign title', 'uncanny-automator' ),
				'tokenType' => 'text',
			),
			array(
				'tokenId'   => 'CAMPAIGN_ID',
				'tokenName' => __( 'Campaign ID', 'uncanny-automator' ),
				'tokenType' => 'text',
			),
			array(
				'tokenId'   => 'CAMPAIGN_CREATOR_EMAIL',
				'tokenName' => __( 'Campaign creator email', 'uncanny-automator' ),
				'tokenType' => 'email',
			),
			// Donation tokens.
			array(
				'tokenId'   => 'DONATION_TITLE',
				'tokenName' => __( 'Donation title', 'uncanny-automator' ),
				'tokenType' => 'text',
			),
			array(
				'tokenId'   => 'DONATION_ID',
				'tokenName' => __( 'Donation ID', 'uncanny-automator' ),
				'tokenType' => 'int',
			),
			array(
				'tokenId'   => 'RECURRING_AMOUNT',
				'tokenName' => __( 'Recurring amount', 'uncanny-automator' ),
				'tokenType' => 'int', //round?
			),
			array(
				'tokenId'   => 'DONATION_DONOR_NAME',
				'tokenName' => __( 'Donor name', 'uncanny-automator' ),
				'tokenType' => 'text',
			),
			array(
				'tokenId'   => 'DONATION_DONOR_EMAIL',
				'tokenName' => __( 'Donor email', 'uncanny-automator' ),
				'tokenType' => 'email',
			),
		);
	}

	/**
	 * Populate Reccuring Donation Token Values.
	 *
	 * @param int $donation_id Donation ID.
	 *
	 * @return array
	 */
	public function hydrate_recurring_donation_tokens( $donation_id ) {

		// Generate array of empty default values.
		$defaults = wp_list_pluck( $this->get_donation_tokens_config(), 'tokenId' );
		$tokens   = array_fill_keys( $defaults, '' );

		// Get Charitable_Donation object.
		$donation = $this->get_donation( $donation_id );
		// Bail invalid donation ID.
		if ( ! $donation ) {
			return $tokens;
		}

		// Donation Data.
		$tokens['DONATION_ID']    = $donation_id;
		$tokens['DONATION_TITLE'] = get_the_title( $donation_id );

		//TODO REVIEW is this the correct recurring amount
		$tokens['RECURRING_AMOUNT'] = $donation->get_amount_formatted();

		// Donor Data.
		$donor_data = $donation->get_donor_data();
		if ( ! empty( $donor_data ) ) {
			$tokens['DONATION_DONOR_NAME']  = $donor_data['name'];
			$tokens['DONATION_DONOR_EMAIL'] = $donor_data['email'];
		}

		// Campaign Data.
		$campaign = $this->get_donation_campaign( $donation );
		// Bail invalid campaign.
		if ( ! $campaign ) {
			return $tokens;
		}

		$tokens['CAMPAIGN_ID']            = $campaign->get_campaign_id();
		$tokens['CAMPAIGN_TITLE']         = $campaign->post_title;
		$tokens['CAMPAIGN_CREATOR_EMAIL'] = $campaign->get_campaign_creator_email();

		return $tokens;
	}

}
